/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components;

import static com.google.gwt.query.client.GQuery.$;

import org.dussan.vaadin.dtable.client.VDTable;
import org.dussan.vaadin.dtable.client.components.base.TableComponent;
import org.dussan.vaadin.dtable.client.components.events.SplitterEvent;

import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.Style.Cursor;
import com.google.gwt.dom.client.Style.Display;
import com.google.gwt.query.client.Function;
import com.google.gwt.query.client.GQuery;
import com.google.gwt.query.client.css.CSS;
import com.google.gwt.query.client.css.Length;
import com.google.gwt.user.client.Event;

public class Splitter
    extends TableComponent
{

    private String mainId = "div#v-dtable";

    private String frozenBodyId = "table#dt-frozen-body-table";

    private String bodyId = "table#dt-body-table";

    private String mainContainerWrapperId = "div#dt-main-container-wrapper";

    private String mainContainerId = "table#dt-main-container";

    private String splitterId = "#dt-splitter";

    private String splitterBackgroundId = "div#dt-splitter-background";

    private String splitterContainerId = "td#dt-splitter-container";

    final String[][] filter = { { "td#dt-frozen-header-wrapper", "td#dt-frozen-body-wrapper" },
        { "div#dt-frozen-header-container", "div#dt-frozen-body-container" } };

    private boolean showSplitter = false;

    private int mouseX = 0;

    private int splitterLeft = 0;

    private GQuery clickEventHandler = null;

    private GQuery moveEventHandler = null;

    public Splitter( VDTable dTable )
    {
        super( dTable );
    }

    private Splitter getThis()
    {
        return this;
    }

    private void showSplitter( boolean show )
    {
        final String[][] filter = { { "td#dt-frozen-header-wrapper", "td#dt-frozen-body-wrapper" },
            { "div#dt-frozen-header-container", "div#dt-frozen-body-container" } };
        GQuery mainContainer = $( mainId ).find( mainContainerId );

        if ( show && !showSplitter )
        {
            showSplitter = show;
            mainContainer.children( "tr" )
                         .children( splitterContainerId )
                         .css( "width", "" )
                         .css( "display", "" );
            mainContainer.children( "tr" )
                         .children( filter[0] )
                         .css( CSS.WIDTH.with( Length.px( getDTable().getFrozenTableWidth() ) ) );
            mainContainer.children( "tr" )
                         .children( "td" )
                         .children( filter[1] )
                         .css( CSS.WIDTH.with( Length.px( getDTable().getFrozenTableWidth() ) ) );

            // set width to 100% - Chrome want this
            mainContainer.children( "tr" )
                         .children( "td" )
                         .children( filter[1] )
                         .css( CSS.WIDTH.with( Length.pct( 100 ) ) );
        }
        else if ( !show && showSplitter )
        {
            showSplitter = show;
            mainContainer.children( "tr" )
                         .children( splitterContainerId )
                         .css( CSS.WIDTH.with( Length.px( 0 ) ) )
                         .css( CSS.DISPLAY.with( Display.NONE ) );
            mainContainer.children( "tr" )
                         .children( filter[0] )
                         .css( CSS.WIDTH.with( Length.px( 0 ) ) );
            mainContainer.children( "tr" )
                         .children( "td" )
                         .children( filter[1] )
                         .css( CSS.WIDTH.with( Length.px( 0 ) ) );
        }
        activateClickEventHandler( show );
    }

    private void activateClickEventHandler( boolean activate )
    {
        if ( activate )
        {
            activateClickEventHandler( false );
            activateMoveEventHandler( false );
            clickEventHandler = $( mainId ).find( mainContainerWrapperId )
                                           .on( "mousedown", "td", new Function()
                                           {
                                               public boolean f( Event event )
                                               {
                                                   return SplitterEvent.mousedown( getThis(), event );
                                               }
                                           } );
        }
        else if ( clickEventHandler != null )
        {
            clickEventHandler.off( "mousedown", "td" );
            clickEventHandler = null;
        }
    }

    public void activateMoveEventHandler( boolean activate )
    {
        if ( activate )
        {
            activateMoveEventHandler( false );
            moveEventHandler = $( mainId ).on( "mousemove", "div", new Function()
            {
                public void f( Element e )
                {
                    SplitterEvent.mousemove( getThis(), e.getId(), getEvent().getClientX() );
                }
            } )
                                          .on( "mouseup", "div", new Function()
                                          {
                                              public void f( Element e )
                                              {
                                                  SplitterEvent.mouseup( getThis(), e.getId(),
                                                                         getEvent().getClientX() );
                                              }
                                          } )
                                          .on( "mouseout", "div", new Function()
                                          {
                                              public void f( Element e )
                                              {
                                                  SplitterEvent.mouseout( getThis(), e.getId() );
                                              }
                                          } );
        }
        else if ( moveEventHandler != null )
        {
            moveEventHandler.off( "mousemove", "div" )
                            .off( "mouseup", "div" )
                            .off( "mouseout", "div" );
            moveEventHandler = null;
        }
    }

    public String getFrozenBodyId()
    {
        return frozenBodyId;
    }

    public String getBodyId()
    {
        return bodyId;
    }

    public String[][] getFilter()
    {
        return filter;
    }

    public String getMainId()
    {
        return mainId;
    }

    public String getMainContainerId()
    {
        return mainContainerId;
    }

    public String getMainContainerWrapperId()
    {
        return mainContainerWrapperId;
    }

    public String getSplitterId()
    {
        return splitterId;
    }

    public String getSplitterContainerId()
    {
        return splitterContainerId;
    }

    public String getSplitterBackgroundId()
    {
        return splitterBackgroundId;
    }

    public int getMouseX()
    {
        return mouseX;
    }

    public void setMouseX( int mouseX )
    {
        this.mouseX = mouseX;
    }

    public int getSplitterLeft()
    {
        return splitterLeft;
    }

    public void setSplitterLeft( int splitterLeft )
    {
        this.splitterLeft = splitterLeft;
    }

    @Override
    public void setIds()
    {
        if ( !mainId.endsWith( "" + getDTable().getId() ) )
        {
            mainId += getDTable().getId();
            frozenBodyId += getDTable().getId();
            bodyId += getDTable().getId();
        }
    }

    @Override
    public void load()
    {
        setIds();
        showSplitter( getDTable().config()
                                 .isFrozenTableEnabled() );
    }

    @Override
    public void unload()
    {
        setEventsHandlerDisabled();
    }

    @Override
    public void setEventsHandlerEnabled()
    {
        boolean activate = ( getDTable().config()
                                        .isHideFrozenTableEnabled()
            || getDTable().config()
                          .isResizeFrozenTableEnabled() )
            && getDTable().config()
                          .isFrozenTableEnabled()
            && !getDTable().isFrozenTableEmpty();
        activateClickEventHandler( activate );
        $( splitterContainerId ).css( CSS.CURSOR.with( activate ? Cursor.COL_RESIZE : Cursor.DEFAULT ) );
    }

    @Override
    public void setEventsHandlerDisabled()
    {
        activateClickEventHandler( false );
        activateMoveEventHandler( false );
    }

}
