/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.processors;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
import org.apache.commons.compress.archivers.tar.TarArchiveOutputStream;
import org.apache.commons.compress.compressors.bzip2.BZip2CompressorOutputStream;
import org.apache.commons.compress.compressors.gzip.GzipCompressorOutputStream;
import org.apache.commons.compress.utils.IOUtils;
import org.dussan.vaadin.dtable.DTable;
import org.dussan.vaadin.dtable.windows.exporter.base.Exporter;

public class CompressingProcessor
{

    public static final float tarBz2( File exportedFile, List<File> files )
    {
        try
        {
            FileOutputStream fileOutputStream = new FileOutputStream( exportedFile );
            BufferedOutputStream bufferedOutputStream = new BufferedOutputStream( fileOutputStream );
            BZip2CompressorOutputStream bzip2CompressorOutputStream =
                new BZip2CompressorOutputStream( bufferedOutputStream );
            TarArchiveOutputStream tarArchiveOutputStream = new TarArchiveOutputStream( bzip2CompressorOutputStream );
            for ( File file : files )
            {
                TarArchiveEntry tarArchiveEntry = new TarArchiveEntry( file.getName() );
                tarArchiveEntry.setSize( file.length() );
                tarArchiveOutputStream.setLongFileMode( TarArchiveOutputStream.LONGFILE_GNU );
                tarArchiveOutputStream.putArchiveEntry( tarArchiveEntry );
                IOUtils.copy( new FileInputStream( file ), tarArchiveOutputStream );
                tarArchiveOutputStream.closeArchiveEntry();
                file.delete();
            }
            tarArchiveOutputStream.finish();
            tarArchiveOutputStream.close();
            bzip2CompressorOutputStream.close();
            bufferedOutputStream.close();
            fileOutputStream.close();
            return Exporter.EXPORT_PREPARED;
        }
        catch ( Exception e )
        {
            Logger.getLogger( DTable.class.getName() )
                  .log( Level.INFO, e.getLocalizedMessage(), e );
            return Exporter.EXPORT_ERROR;
        }
    }

    public static final float tarGz( File exportedFile, List<File> files )
    {
        try
        {
            FileOutputStream fileOutputStream = new FileOutputStream( exportedFile );
            BufferedOutputStream bufferedOutputStream = new BufferedOutputStream( fileOutputStream );
            GzipCompressorOutputStream gzipCompressorOutputStream =
                new GzipCompressorOutputStream( bufferedOutputStream );
            TarArchiveOutputStream tarArchiveOutputStream = new TarArchiveOutputStream( gzipCompressorOutputStream );
            for ( File file : files )
            {
                TarArchiveEntry tarArchiveEntry = new TarArchiveEntry( file.getName() );
                tarArchiveEntry.setSize( file.length() );
                tarArchiveOutputStream.setLongFileMode( TarArchiveOutputStream.LONGFILE_GNU );
                tarArchiveOutputStream.putArchiveEntry( tarArchiveEntry );
                IOUtils.copy( new FileInputStream( file ), tarArchiveOutputStream );
                tarArchiveOutputStream.closeArchiveEntry();
                file.delete();
            }
            tarArchiveOutputStream.finish();
            tarArchiveOutputStream.close();
            gzipCompressorOutputStream.close();
            bufferedOutputStream.close();
            fileOutputStream.close();
            return Exporter.EXPORT_PREPARED;
        }
        catch ( Exception e )
        {
            Logger.getLogger( DTable.class.getName() )
                  .log( Level.INFO, e.getLocalizedMessage(), e );
            return Exporter.EXPORT_ERROR;
        }
    }

    public static final float zip( File exportedFile, List<File> files )
    {
        try
        {
            ZipOutputStream zipOutputStream = new ZipOutputStream( new FileOutputStream( exportedFile ) );
            for ( File file : files )
            {
                FileInputStream fileInputStream = new FileInputStream( file );
                zipOutputStream.putNextEntry( new ZipEntry( file.getName() ) );
                int len;
                byte[] buf = new byte[1024];
                while ( ( len = fileInputStream.read( buf ) ) > 0 )
                {
                    zipOutputStream.write( buf, 0, len );
                }
                zipOutputStream.closeEntry();
                fileInputStream.close();
                file.delete();
            }
            zipOutputStream.finish();
            zipOutputStream.close();
            return Exporter.EXPORT_PREPARED;
        }
        catch ( Exception e )
        {
            Logger.getLogger( DTable.class.getName() )
                  .log( Level.INFO, e.getLocalizedMessage(), e );
            return Exporter.EXPORT_ERROR;
        }
    }

}
