/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.processors;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.dussan.vaadin.dtable.DTable;
import org.dussan.vaadin.dtable.properties.order.Order;
import org.dussan.vaadin.dtable.properties.order.OrderType;

public class OrderProcessor
{

    public static Order[] process( Object orderObjects, String[] headers )
    {
        if ( orderObjects != null && headers != null )
        {
            List<Order> orderColumns = new ArrayList<>();
            for ( String order : orderObjects.toString()
                                             .split( "_:_" ) )
            {
                try
                {
                    int column = Integer.parseInt( order.split( " " )[0] );
                    orderColumns.add( new Order( column, headers[column], OrderType.type( order.split( " " )[1] ) ) );
                }
                catch ( Exception e )
                {
                    Logger.getLogger( DTable.class.getName() )
                          .log( Level.INFO, e.getLocalizedMessage(), e );
                    return null;
                }
            }
            return orderColumns.toArray( new Order[] {} );
        }
        return null;
    }

    public static Order[] process( Order[] orders, String[] headers )
    {
        if ( orders != null && headers != null )
        {
            List<Order> orderColumns = new ArrayList<>();
            for ( Order order : orders )
            {
                int column = order.getColumn();
                orderColumns.add( new Order( column, headers[column], order.getType() ) );
            }
            return orderColumns.toArray( new Order[] {} );
        }
        return null;
    }

}
