/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.processors;

import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;

import org.dussan.vaadin.dtable.config.formatting.Formatting;
import org.dussan.vaadin.dtable.properties.filter.Filter;
import org.dussan.vaadin.dtable.properties.search.Search;

public class SearchProcessor
{

    private static String getOperator( String value, Formatting formatting )
    {
        if ( value != null && !value.isEmpty() )
        {
            String tempValue = value.toUpperCase()
                         .trim();
            if ( !tempValue.isEmpty() )
            {
                if ( formatting != null && ( formatting.getFormat() instanceof NumberFormat
                    || formatting.getFormat() instanceof DecimalFormat || formatting.getFormat() instanceof DateFormat
                    || formatting.getFormat() instanceof SimpleDateFormat ) )
                {
                    if ( tempValue.startsWith( "<=" ) || tempValue.startsWith( ">=" ) || tempValue.startsWith( "<>" ) )
                    {
                        return tempValue.substring( 0, 2 )
                                        .trim();
                    }
                    else if ( tempValue.startsWith( ">" ) || tempValue.startsWith( "<" ) || tempValue.startsWith( "=" ) )
                    {
                        return tempValue.substring( 0, 1 )
                                        .trim();
                    }
                    return "=";
                }
                else
                {
                    if ( tempValue.startsWith( "NOT LIKE " ) )
                    {
                        return "NOT LIKE";
                    }
                    return "LIKE";
                }
            }
        }
        return null;
    }

    private static String getValue( String value, String operator, Formatting formatting )
    {
        if ( value != null && !value.isEmpty() )
        {
            String tempValue = value.trim();
            if ( !tempValue.isEmpty() )
            {
                if ( formatting != null && ( formatting.getFormat() instanceof NumberFormat
                    || formatting.getFormat() instanceof DecimalFormat || formatting.getFormat() instanceof DateFormat
                    || formatting.getFormat() instanceof SimpleDateFormat ) )
                {
                    return !tempValue.startsWith( operator ) ? tempValue : tempValue.substring( operator.length() )
                                                                                    .trim();
                }
                else
                {
                    String tempOperator = operator + " ";
                    if ( tempValue.startsWith( tempOperator ) )
                    {
                        tempValue = tempValue.substring( tempOperator.length() )
                                             .trim();
                    }
                    tempValue = tempValue.startsWith( "\"" ) ? tempValue.substring( 1 ) : tempValue;
                    tempValue = tempValue.startsWith( "'" ) ? tempValue.substring( 1 ) : tempValue;
                    tempValue = tempValue.endsWith( "\"" ) ? tempValue.substring( 0, tempValue.length() - 1 ) : tempValue;
                    tempValue = tempValue.endsWith( "'" ) ? tempValue.substring( 0, tempValue.length() - 1 ) : tempValue;
                    return tempValue;
                }
            }
        }
        return null;
    }

    public static Search process( String[] search, String header, Formatting formatting )
    {
        if ( search != null && search.length == 2 )
        {
            Integer column = Integer.parseInt( search[0].trim() );
            String operator = getOperator( search[1].trim(), formatting );
            if ( operator == null )
            {
                return null;
            }

            String value = getValue( search[1], operator, formatting );
            if ( value == null )
            {
                return null;
            }

            String localeOriginalValue = getValue( search[1], operator, formatting );
            Object originalValue = FormattingProcessor.process( formatting, value );
            value = originalValue == null ? value : FormattingProcessor.process( formatting, originalValue );

            return new Search( column, header, operator.trim(), value.trim(),
                               originalValue == null ? null : originalValue.toString()
                                                                           .trim(),
                               originalValue == null ? null : localeOriginalValue.trim() );
        }
        return null;
    }

    public static Filter search2filter( Search search )
    {
        return new Filter( search.getColumn(), search.getHeader(), new String[] { search.getOperator() },
                           new String[] { search.getValue() }, new String[] { search.getOriginalValue() } );
    }

}
