/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.translations;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.dussan.vaadin.dtable.DTable;

public class Translation
    implements Serializable
{

    private static final long serialVersionUID = -6496946751885167174L;

    private Map<Locale, Map<String, String>> translatedWords = new HashMap<>();

    public Translation()
    {
        Map<String, String> translatedWords = new HashMap<>();
        translatedWords.put( TranslatedWords.ACTION.toString(), "ACTION" );
        translatedWords.put( TranslatedWords.ASCENDING_ORDER_FOR_COLUMN.toString(), "ASCENDING ORDER FOR COLUMN" );
        translatedWords.put( TranslatedWords.AT_LEAST_ONE_COLUMN_HAVE_TO_BE_UNHIDDEN.toString(),
                             "At least one column have to be unhidden!" );
        translatedWords.put( TranslatedWords.AT_LEAST_ONE_FROZEN_COLUMN_HAVE_TO_BE_UNHIDDEN.toString(),
                             "At least one frozen column have to be unhidden!" );
        translatedWords.put( TranslatedWords.AT_LEAST_ONE_COLUMN_HAVE_TO_BE_FROZEN_AND_UNHIDDEN.toString(),
                             "At least one column have to be frozen and unhidden!" );
        translatedWords.put( TranslatedWords.AT_LEAST_ONE_COLUMN_HAVE_TO_BE_UNFROZEN_AND_UNHIDDEN.toString(),
                             "At least one column have to be unfrozen and unhidden!" );
        translatedWords.put( TranslatedWords.AVAILABLE_COLUMNS.toString(), "Available columns" );
        translatedWords.put( TranslatedWords.CLOSE.toString(), "CLOSE" );
        translatedWords.put( TranslatedWords.COLUMNS_THAT_CAN_NOT_BE_FROZEN_OR_UNFROZEN_ARE_DISABLED.toString(),
                             "Columns that can not be frozen or unfrozen, are disabled." );
        translatedWords.put( TranslatedWords.COLUMNS_THAT_CAN_NOT_BE_HIDDEN_OR_UNHIDDEN_ARE_DISABLED.toString(),
                             "Columns that can not be hidden or unhidden, are disabled." );
        translatedWords.put( TranslatedWords.CSV_FILE.toString(), "CSV file" );
        translatedWords.put( TranslatedWords.DATA_ARE_READY_FOR_DOWNLOAD.toString(), "Data are ready for download!" );
        translatedWords.put( TranslatedWords.DATA_ERROR.toString(), "DATA ERROR" );
        translatedWords.put( TranslatedWords.DESCENDING_ORDER_FOR_COLUMN.toString(), "DESCENDING ORDER FOR COLUMN" );
        translatedWords.put( TranslatedWords.DOWNLOAD_THE_DATA.toString(), "DOWNLOAD THE DATA" );
        translatedWords.put( TranslatedWords.ERROR.toString(), "ERROR" );
        translatedWords.put( TranslatedWords.EXPORT.toString(), "EXPORT" );
        translatedWords.put( TranslatedWords.EXPORTED_COLUMN_DATA_TYPE_AND_COLUMN_DEFINITION_TYPE_ARE_NOT_THE_SAME.toString(),
                             "Exported column data type and column definition type are not the same!" );
        translatedWords.put( TranslatedWords.EXPORTING_DATA_TO_CSV_FILE.toString(), "Exporting data (CSV file)" );
        translatedWords.put( TranslatedWords.EXPORTING_DATA_TO_MS_ACCESS_FILE.toString(),
                             "Exporting data (MS Access file)" );
        translatedWords.put( TranslatedWords.EXPORTING_DATA_TO_MS_EXCEL_FILE.toString(),
                             "Exporting data (MS Excel file)" );
        translatedWords.put( TranslatedWords.EXPORTING_DATA_TO_PDF_FILE.toString(), "Exporting data (PDF file)" );
        translatedWords.put( TranslatedWords.FILTERS.toString(), "FILTERS:" );
        translatedWords.put( TranslatedWords.FREEZE_COLUMN.toString(), "FREEZE COLUMN" );
        translatedWords.put( TranslatedWords.FREEZE_UNFREEZE_COLUMNS.toString(), "Freeze / Unfreeze columns" );
        translatedWords.put( TranslatedWords.FROZEN.toString(), "frozen" );
        translatedWords.put( TranslatedWords.FROZEN_COLUMNS.toString(), "Frozen columns" );
        translatedWords.put( TranslatedWords.HIDE_COLUMN.toString(), "HIDE COLUMN" );
        translatedWords.put( TranslatedWords.HIDDEN.toString(), "hidden" );
        translatedWords.put( TranslatedWords.HIDDEN_COLUMNS.toString(), "Hidden columns" );
        translatedWords.put( TranslatedWords.HIDE_UNHIDE_COLUMNS.toString(), "Hide / Unhide columns" );
        translatedWords.put( TranslatedWords.IN_PREPARING_DATA_AN_ERROR_OCCURRED.toString(),
                             "In preparing data an error occurred!" );
        translatedWords.put( TranslatedWords.INCLUDE_HIDDEN_COLUMNS.toString(), "Include hidden columns" );
        translatedWords.put( TranslatedWords.INFO.toString(), "INFO" );
        translatedWords.put( TranslatedWords.IS_EMPTY.toString(), "IS EMPTY" );
        translatedWords.put( TranslatedWords.IS_NOT_EMPTY.toString(), "IS NOT EMPTY" );
        translatedWords.put( TranslatedWords.MAXIMUM_PERMITTED_NUMBER_OF_FROZEN_COLUMNS_IS.toString(),
                             "Maximum permitted number of frozen columns is " );
        translatedWords.put( TranslatedWords.MS_ACCESS_FILE.toString(), "MS Access file" );
        translatedWords.put( TranslatedWords.MS_ACCESS_PARTIAL_TABLE_NAME.toString(), "_part" );
        translatedWords.put( TranslatedWords.MS_EXCEL_FILE.toString(), "MS Excel file" );
        translatedWords.put( TranslatedWords.MS_EXCEL_PARTIAL_SHEET_NAME.toString(), " - Part " );
        translatedWords.put( TranslatedWords.NO_DATA_TO_DISPLAY.toString(), "No data to display!" );
        translatedWords.put( TranslatedWords.NO_DATA_FOR_EXPORT.toString(), "No data for export!" );
        translatedWords.put( TranslatedWords.NO_NEW_DATA.toString(), "NO NEW DATA" );
        translatedWords.put( TranslatedWords.OK.toString(), "OK" );
        translatedWords.put( TranslatedWords.PDF_FILE.toString(), "PDF file" );
        translatedWords.put( TranslatedWords.PREPARING_DATA.toString(), "Preparing data" );
        translatedWords.put( TranslatedWords.PREPARING_DATA_FOR_DOWNLOAD.toString(), "Preparing data for download..." );
        translatedWords.put( TranslatedWords.PREPARING_DATA_IS_COMPLETED.toString(), "Preparing data is completed!" );
        translatedWords.put( TranslatedWords.PREPARING_DATA_IS_STOPPED.toString(), "Preparing data is stopped!" );
        translatedWords.put( TranslatedWords.PREPARING_DATA_PLEASE_WAIT.toString(), "Preparing data, please wait..." );
        translatedWords.put( TranslatedWords.QUICK_SEARCH.toString(), "Quick search" );
        translatedWords.put( TranslatedWords.REMOVE_ALL_ORDERS.toString(), "REMOVE ALL ORDERS" );
        translatedWords.put( TranslatedWords.REMOVE_FILTERS_FROM_COLUMN.toString(), "REMOVE FILTERS FROM COLUMN" );
        translatedWords.put( TranslatedWords.REMOVE_ORDER_FOR_COLUMN.toString(), "REMOVE ORDER FOR COLUMN" );
        translatedWords.put( TranslatedWords.REMOVE_ORDERS_FROM_COLUMNS.toString(), "Remove orders from columns" );
        translatedWords.put( TranslatedWords.SEARCH.toString(), "SEARCH" );
        translatedWords.put( TranslatedWords.SEARCH_IN_COLUMNS.toString(), "Search in columns" );
        translatedWords.put( TranslatedWords.SIZE_OF_THE_REQUESTED_DATA_IS_ZERO.toString(),
                             "Size of the requested data is zero!" );
        translatedWords.put( TranslatedWords.SOME_DATA_ARE_NOT_EXPORTED_BECAUSE_THEY_EXCEEDED_THE_ALLOWED_QUOTA_OF_ROWS_WHICH_IS_ALLOWED_IN_MS_EXCEL.toString(),
                             "Some data are not exported because they exceeded the allowed quota of rows, which is allowed in MS Excel." );
        translatedWords.put( TranslatedWords.SOMETHING_IS_WRONG_IN_DATA_PREPARATION_PLEASE_CHECK_YOUR_DATA_SETTINGS.toString(),
                             "Something is wrong in data preparation, please check your data settings!" );
        translatedWords.put( TranslatedWords.STOP_PREPARING_DATA.toString(), "STOP PREPARING DATA" );
        translatedWords.put( TranslatedWords.TO_EXPORT_THE_DATA_PRESS_THE_BUTTON_EXPORT.toString(),
                             "To export the data press the button \"EXPORT\"!" );
        translatedWords.put( TranslatedWords.UNFREEZE_COLUMN.toString(), "UNFREEZE COLUMN" );
        translatedWords.put( TranslatedWords.UNFROZEN_COLUMNS.toString(), "Unfrozen columns" );
        translatedWords.put( TranslatedWords.UNHIDDEN_COLUMNS.toString(), "Unhidden columns" );
        translatedWords.put( TranslatedWords.WHEN_TRANSMITTING_DATA_AN_ERROR_HAS_OCCURRED.toString(),
                             "When transmitting data an error has occurred!" );
        translatedWords.put( TranslatedWords.WINDOW_FOR_EXPORTING_DATA_IS_ALREADY_OPEN.toString(),
                             "Window for exproting data is already open!" );
        translatedWords.put( TranslatedWords.WINDOW_FOR_HIDING_UNHIDING_COLUMNS_DATA_IS_ALREADY_OPEN.toString(),
                             "Window for hiding/unhiding columns is already open!" );

        // save translations
        this.translatedWords.put( new Locale( "en" ), translatedWords );
    }

    public String get( Locale locale, TranslatedWords translatedWord )
    {
        try
        {
            if ( translatedWords.containsKey( locale ) && translatedWords.get( locale )
                                                                         .containsKey( translatedWord.toString() ) )
            {
                return translatedWords.get( locale )
                                      .get( translatedWord.toString() )
                                      .toString();
            }
            return translatedWords.get( new Locale( "en" ) )
                                  .get( translatedWord.toString() )
                                  .toString();
        }
        catch ( Exception e )
        {
            Logger.getLogger( DTable.class.getName() )
                  .log( Level.INFO, e.getLocalizedMessage(), e );
        }
        return "_TRANSLATION_ERROR_";
    }

    public void set( Locale locale, TranslatedWords translateWord, String translatedWord )
    {
        if ( locale != null && translatedWord != null && !translatedWord.isEmpty() )
        {
            Map<String, String> translatedWords = get( locale );
            if ( translatedWords == null )
            {
                translatedWords = new HashMap<>();
            }
            translatedWords.put( translateWord.toString(), translatedWord );
            this.translatedWords.put( locale, translatedWords );
        }
    }

    public Map<String, String> get( Locale locale )
    {
        if ( translatedWords.containsKey( locale ) )
        {
            translatedWords.get( locale );
        }
        return translatedWords.get( new Locale( "en" ) );
    }

    public void set( Locale locale, Map<String, String> translatedWords )
    {
        if ( locale != null && translatedWords != null && !translatedWords.isEmpty() )
        {
            this.translatedWords.put( locale, translatedWords );
        }
    }

    public Map<String, String> getData( Locale locale )
    {
        if ( locale != null && translatedWords.get( locale ) != null && !translatedWords.get( locale )
                                                                                        .isEmpty() )
        {
            return translatedWords.get( locale );
        }
        return translatedWords.get( new Locale( "en" ) );
    }

}
