/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.windows.exporter.type;

public enum CompressionType
{

    NONE( "" ), TAR_BZ2( "tar.bz2" ), TAR_GZ( "tar.gz" ), ZIP( "zip" );

    private String type;

    /**
     * Set compression type string.
     * 
     * @param type compression type string
     */
    private CompressionType( String type )
    {
        this.type = type;
    }

    /**
     * Get compression type extension for specified enum.
     * 
     * @return compression type extension for specified enum
     */
    public String getExtension()
    {
        return "." + type;
    }

    /**
     * Get the value for specified compression type enum.
     * 
     * @return the value for specified compression type enum
     */
    public String getType()
    {
        return type;
    }

    @Override
    public String toString()
    {
        return "compression type: " + name();
    }

}
